// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pro Rata Volume is an estimate of what the Bar's final volume will be 
	/// </summary>
	[Description("Pro Rata Volume is an estimate of what the Bar's final Volume will be")]
	[Gui.Design.DisplayName("VolumePRV")]
	public class VolumePRV : Indicator
	{
		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// This indicator produce bars just like Volume_MultiColor
		/// </summary>

	#region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
		private char		tab = '\u0020';
		private string		msg = "PRV Off";
		private double 		PRVol = 0;
		private int			widthV = 5;
	#endregion
		
		protected override void Initialize()
		{
			DisplayInDataBox = false; 
			Add(new Plot(new Pen(Color.Black, widthV), PlotStyle.Bar, "VolumeUp"));  // 0
			Add(new Plot(new Pen(Color.Red, widthV), PlotStyle.Bar, "VolumeDown"));  // 1
			Add(new Plot(new Pen(Color.MediumOrchid, widthV), PlotStyle.Bar, "VolumeEQ"));  // 2
			Add(new Line(Color.DarkGray, 0, "Zero line"));
			DisplayInDataBox = true; 
			
			Add(new Plot(new Pen(Color.Gold, widthV-1), PlotStyle.Bar, "PRVu"));  // 3
			Add(new Plot(new Pen(Color.Pink, widthV-1), PlotStyle.Bar, "PRVd"));  // 4
						
			ChartOnly		= true;
			Overlay			= false;
			CalculateOnBarClose 	= false;
			DrawOnPricePanel 	= false;
			PaintPriceMarkers 	= true;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
	        if (FirstTickOfBar & CurrentBar >= 1)   // At least one bar complete...
		    { 
				Values[3].Set(1,Volume[1]);		// hide old PRV plots below Volume plot values
				Values[4].Set(1,Volume[1]);
				// Draws text in the upper right corner of panel 1 
				DrawTextFixed("Vol1", "Prior Volume =" + Volume[1].ToString(), TextPosition.TopRight);
      		} 

			if (Bars.PercentComplete < .02) PRVol = Volume[0] ;
			else PRVol = Volume[0]/Bars.PercentComplete - (Volume[0]/Bars.PercentComplete % 1);
			//  using modulo to truncate into a whole number, another way below? 
			//  PRVol = Math.Floor(Volume[0]/Bars.PercentComplete);
			
			if (SessionActive())
			{
				//Values[3].Set(PRVol);  				// Plot PRV first so it is underneath Volume
				if (CurrentBar<1)  Values[3].Set(PRVol);  // 1st bar, just get something out there
				else if (PRVol>Volume[1]) 
					 Values[3].Set(PRVol);  // 3=up
				else Values[4].Set(PRVol);	// 4=down
				// DrawDot("PRVball", true, 0, PRVol, Color.Purple);
				// Note DrawText doesn't like barsago<0 so pad string with blanks to move it over
				msg = "PRV = ".PadLeft(35,tab);
				DrawText("PRVnum", msg + PRVol.ToString(), 0, PRVol+200, Color.Black);
				//DrawText("PRVnum", false, "PRV = " + PRVol.ToString(), 0, PRVol+200, Color.Black, null, StringAlignment.Near, Color.Empty, Color.Empty, 0);

			}
			else
			{
				// RemoveDrawObject("PRVball");
				msg = "PRV Off".PadLeft(25,tab);
				DrawText("PRVnum", msg, 0, PRVol, Color.Black);
			}

			//  Regular TriColor Volume plot on top of PRV plot
			if (CurrentBar < 1)						// First bar?
				Values[2].Set(Volume[0]);			// Plot it as an "equal" volume.
			
			else if (Close[0] > Close[1])			// Is PRICE up over last bar?
				Values[0].Set(Volume[0]);			// Yes.  Plot it as an "up" volume.

			else if (Close[0] < Close[1])
				Values[1].Set(Volume[0]);			// Yes.  Plot it as an "down" volume.
	
			else 
				Values[2].Set(Volume[0]);			// Yes.  Plot it as an "equal" volume.					
		}
        #region Properties
		/*
		[Description("Volume Bar width")]
		[Category(" ")]
		[Gui.Design.DisplayNameAttribute("Volume Bar width")]
        public int WidthV
        {
           get { return widthV; }
           set { widthV = Math.Max(2, value); }  // min=2
        }
		*/
		
		/*
		[Browsable(false)]		// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
		[XmlIgnore()]			// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public double PRV
		{
 		   get { Update(); return PRVol; }
		}	
		*/
	
        #endregion

        #region Miscellaneous
	    public bool SessionActive()
        {
			if (ChartControl != null
					&& Bars != null
					&& Bars.Count > 0
					&& Bars.MarketData != null
					&& Bars.MarketData.Connection.PriceStatus == Cbi.ConnectionStatus.Connected
					&& (Bars.MarketData.Connection.Options.Provider != Cbi.Provider.OpenTick || !(Bars.MarketData.Connection.Options as Cbi.OpenTickOptions).UseDelayedData)
					&& Bars.Session.InSession(Now, Bars.Period.Id, true))
				return true;
            return false;
        }
   		public DateTime Now
		{
			get { return (Bars.MarketData.Connection.Options.Provider == Cbi.Provider.Replay ? Bars.MarketData.Connection.Now : DateTime.Now); }
		}

 		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VolumePRV[] cacheVolumePRV = null;

        private static VolumePRV checkVolumePRV = new VolumePRV();

        /// <summary>
        /// Pro Rata Volume is an estimate of what the Bar's final Volume will be
        /// </summary>
        /// <returns></returns>
        public VolumePRV VolumePRV()
        {
            return VolumePRV(Input);
        }

        /// <summary>
        /// Pro Rata Volume is an estimate of what the Bar's final Volume will be
        /// </summary>
        /// <returns></returns>
        public VolumePRV VolumePRV(Data.IDataSeries input)
        {

            if (cacheVolumePRV != null)
                for (int idx = 0; idx < cacheVolumePRV.Length; idx++)
                    if (cacheVolumePRV[idx].EqualsInput(input))
                        return cacheVolumePRV[idx];

            VolumePRV indicator = new VolumePRV();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            VolumePRV[] tmp = new VolumePRV[cacheVolumePRV == null ? 1 : cacheVolumePRV.Length + 1];
            if (cacheVolumePRV != null)
                cacheVolumePRV.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheVolumePRV = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Pro Rata Volume is an estimate of what the Bar's final Volume will be
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumePRV VolumePRV()
        {
            return _indicator.VolumePRV(Input);
        }

        /// <summary>
        /// Pro Rata Volume is an estimate of what the Bar's final Volume will be
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumePRV VolumePRV(Data.IDataSeries input)
        {
            return _indicator.VolumePRV(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Pro Rata Volume is an estimate of what the Bar's final Volume will be
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumePRV VolumePRV()
        {
            return _indicator.VolumePRV(Input);
        }

        /// <summary>
        /// Pro Rata Volume is an estimate of what the Bar's final Volume will be
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumePRV VolumePRV(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VolumePRV(input);
        }

    }
}
#endregion
